# Load libraries
library(hivEstimatesAccuracy)
library(data.table)
library(ggplot2)

# Settings

# Color palette for the plots
# First color: background of confidence interval for the estimated count
# Second color: estimated count
# Third color: reported count
colorPalette <- c("#AAAAAA", "#E69F00", "#56B4E9")

# Separator used for creating a composite of stratum columns. Should not occur in the stratum values.
stratSep <- "_"

# Stratifiation columns
stratVarsNames <- c("Gender", "Transmission", "Migr")

# Start year
startYear <- 2000L

# End quarter
endQrt <- 2017.25

# Input data path
inputDataFilePath <- "d:/Drive/Projects/19. PZH/Scripts/Received/20171013/InputData/EL.csv"
# inputDataFilePath <- "C:/Users/mrosinska/Documents/projekty/ecdc adjustment/data2017/EL.csv"

# A) READ INPUT DATA ------------------------------------------------------------------------------

# Read input data
originalData <- ReadDataFile(inputDataFilePath)

# Get preliminary attributes mapping
attrMapping <- GetPreliminaryAttributesMapping(originalData)
# Adjust mapping
attrMapping[["FirstCD4Count"]] <- "cd4_num"

# Pre-process data
inputData <- ApplyAttributesMapping(originalData, attrMapping)
inputDataValidityInfo <- ValidateInputData(inputData)
inputData <- PreProcessInputData(inputData)

# B) PROCESS DATA ---------------------------------------------------------------------------------

# Create intermediate variables
inputData[, ":="(
  NotificationTime = DateOfNotificationYear + 1/4 * DateOfNotificationQuarter,
  DiagnosisTime = DateOfDiagnosisYear + 1/4 * DateOfDiagnosisQuarter
)]
inputData[, VarX := 4 * (NotificationTime - DiagnosisTime)]
inputData[VarX < 0, VarX := NA]
inputData[, ":="(
  MinNotificationTime = min(NotificationTime),
  MaxNotificationTime = max(NotificationTime)
), by = .(ReportingCountry)]

# Filter
inputData <- inputData[DiagnosisTime > max(startYear, MinNotificationTime)
                       & NotificationTime <= endQrt]

inputData[, ":="(
  VarT = 4 * (min(MaxNotificationTime, endQrt) - DiagnosisTime),
  ReportingDelay = 1L,
  Tf = 4 * (min(MaxNotificationTime,endQrt) - max(startYear, MinNotificationTime))
)]
inputData[, ":="(
  VarXs = Tf - VarX,
  VarTs = Tf - VarT
)]
# NOTE: Otherwise survival model complains
inputData <- inputData[VarXs > VarTs]

# Create a stratum variable - will be used to merge with the estimations,
# takes missing categories as separate categories
inputData[, Stratum := as.factor(do.call(paste, c(.SD, sep = stratSep))),
          .SDcols = stratVarsNames]

# Fit a stratified survival model
model <- inputData[, survival::Surv(time = VarTs,
                                    time2 = VarXs,
                                    event = ReportingDelay)]
fit <- inputData[, survival::survfit(model ~ Stratum)]

# Aggregate and keep only required dimensions
agregat <- inputData[, .(Count = .N),
                     by = .(DiagnosisTime,
                            YearOfDiagnosis = DateOfDiagnosisYear,
                            ReportingCountry,
                            VarT,
                            Stratum)]
# Recreating stratum variables to assign them to the delay distribution dataset
fitStratum <- data.table(
  Delay = fit$time,
  P = fit$surv,
  Var = fit$std.err^2,
  Stratum = factor(rep(seq_along(fit$strata), fit$strata),
                   labels = levels(inputData$Stratum)))
fitStratum[, (stratVarsNames) := tstrsplit(Stratum, stratSep)]
fitStratum[, VarT := max(Delay) - Delay]
fitStratum <- fitStratum[VarT >= 0]
# Merge fit data
agregat <- fitStratum[agregat, on = .(VarT, Stratum)]

# Compute estimated count and its variance
agregat[, ":="(
  EstCount = Count / P,
  EstCountVar = (Count * (Count + 1) / P^4 * Var) + Count * (1 - P) / P^2
)]

# C) PREPARE PLOT DATA ----------------------------------------------------------------------------

# Agragating by year
agregatByYear <- agregat[, .(Count = sum(Count),
                             EstCount = sum(EstCount),
                             EstCountVar = sum(EstCountVar)),
                         by = .(YearOfDiagnosis)]
agregatByYear[, ":="(
  LowerEstCount = EstCount - 1.96 * sqrt(EstCountVar),
  UpperEstCount = EstCount + 1.96 * sqrt(EstCountVar)
)]

# Stratification
colNames <- union(c("YearOfDiagnosis", "Count", "EstCount", "EstCountVar"),
                  stratVarsNames)
# Keep only required columns, convert data to "long" format...
agregatByYearStrat <- melt(agregat[, ..colNames],
                           measure.vars = stratVarsNames,
                           variable.name = "Stratum",
                           value.name = "StratumValue")
# ...and aggregate
agregatByYearStrat <- agregatByYearStrat[, .(Count = sum(Count),
                                             EstCount = sum(EstCount),
                                             EstCountVar = sum(EstCountVar)),
                                         by = .(YearOfDiagnosis, Stratum, StratumValue)]
agregatByYearStrat[, ":="(
  LowerEstCount = EstCount - 1.96 * sqrt(EstCountVar),
  UpperEstCount = EstCount + 1.96 * sqrt(EstCountVar),
  EstCountVar = NULL
)]

# D) PLOT -----------------------------------------------------------------------------------------

# PLOT 1 - total
# Create output plot by Diagnosis year

agregatPlotYr <- ggplot(agregatByYear, aes(x = YearOfDiagnosis)) +
  geom_ribbon(aes(ymin = pmin(pmax(LowerEstCount, 0), max(EstCount)),
                  ymax = pmin(pmax(UpperEstCount, 0), max(EstCount)),
                  fill = "95% confidence interval\nfor estimated total count"),
              alpha = 0.4) +
  scale_fill_manual("Bounds", values = colorPalette[1]) +
  geom_line(aes(y = Count, color = "Reported"), size = 0.2) +
  geom_line(aes(y = EstCount, color = "Estimated total"), size = 0.5) +
  scale_colour_manual("Counts", values = colorPalette[2:3]) +
  ggtitle("Reported and estimated total count of cases") +
  xlab("Diagnosis year") +
  ylab("Count of HIV cases")
print(agregatPlotYr)

# PLOT 2 - split per stratum
plotList <- lapply(stratVarsNames, function(stratum) {
  ggplot(agregatByYearStrat[Stratum == stratum], aes(x = YearOfDiagnosis)) +
    geom_ribbon(aes(ymin = pmin(pmax(LowerEstCount, 0), max(EstCount)),
                    ymax = pmin(pmax(UpperEstCount, 0), max(EstCount)),
                    fill = "95% confidence interval\nfor estimated total count"),
                alpha = 0.4) +
    scale_fill_manual("Bounds", values = colorPalette[1]) +
    geom_line(aes(y = Count, color = "Reported"), size = 0.2) +
    geom_line(aes(y = EstCount, color = "Estimated total"), size = 0.5) +
    scale_colour_manual("Counts", values = colorPalette[2:3]) +
    facet_wrap(~ StratumValue, ncol = 2) +
    ggtitle(paste("Reported and estimated total count of cases, by", stratum)) +
    xlab("Diagnosis year") +
    ylab("Count of HIV cases")
})
PlotMultipleCharts(plotList)
