# Module globals
# NONE

# User interface
dataSummaryUI <- function(id)
{
  ns <- NS(id)

  tagList(
    div(style = "margin-top: 15px"),
    box(
      width = 12,
      title = "Input data overview",
      solidHeader = FALSE,
      status = "primary",
      collapsible = TRUE,
      withSpinner(
        tagList(
          h3("1. Missing data overview"),
          plotOutput(ns("missPlot")),
          h3("2. Observed delay density"),
          plotOutput(ns("delayDensityPlot")),
          h3("3. Observed delay by notification time"),
          plotOutput(ns("meanDelayPlot"))
        ),
        type = 7,
        proxy.height = "50px")
    ),
    box(
      width = 12,
      title = "Validation data summary",
      solidHeader = FALSE,
      status = "warning",
      collapsible = TRUE,
      uiOutput(ns("validSummary"))
    )
  )
}

# Server logic
dataSummary <- function(input, output, session, inputData)
{
  # Get namespace
  ns <- session$ns

  artifacts <- reactive({
    GetDataSummaryArtifacts(inputData())
  })

  output[["missPlot"]] <- renderPlot({
    replayPlot(artifacts()$MissPlot)
  })

  output[["delayDensityPlot"]] <- renderPlot({
    replayPlot(artifacts()$DelayDensityPlot)
  })

  output[["meanDelayPlot"]] <- renderPlot({
    replayPlot(artifacts()$MeanDelayPlot)
  })

  output[["validSummary"]] <- renderUI({
    # Respond to inputData change
    inputData <- inputData()

    isolate({
      inputDataValidityInfo <- ValidateInputData(inputData)

      inputDataValid <- all(sapply(inputDataValidityInfo, "[[", "Valid"))
      wrongValuesCols <- Filter(function(x) length(x$WrongValues) != 0, inputDataValidityInfo)

      headerHTML <- NULL
      textColor <- NULL
      if (inputDataValid) {
        textColor <- "seagreen"
        headerHTML <- p(tags$i(class = "material-icons", "check"),
                        "Input data is valid. No problems have been found.",
                        style = paste("color:", textColor))
      } else {
        textColor <- "red"
        headerHTML <- p(tags$i(class = "material-icons", "error_outline"),
                        "Input data is not valid. Please, correct the following issues:",
                        style = paste("color:", textColor))
      }

      wrongValuesHTML <- NULL
      if (length(wrongValuesCols) > 0) {
        wrongValuesHTML <- tags$li(
          "Incorrect values:",
          tags$ul(
            lapply(names(wrongValuesCols), function(colName) {
              wrongValuesCol <- wrongValuesCols[[colName]]
              tags$li(paste(AddQuoteMarks(colName),
                            "contains invalid value(s)",
                            paste(AddQuoteMarks(wrongValuesCol$WrongValues), collapse = ", ")))
            })
          )
        )
      }

      validationSummaryHTML <- tagList(headerHTML,
                                       tags$ol(wrongValuesHTML,
                                               style = paste("color:", textColor)))

      return(validationSummaryHTML)
    })
  })
}
